#!/usr/bin/python
# -*- coding:utf-8 -*-

""" Central script of DashSTicker widget.
Running this script without arguments gives all available modules.
If you would like to generate HTML of a certain sports score, run thi script as follows

 python loader.py [module_name] [widget_id] [skin]

 widget_id is necessary to avoid conflict between widgets which are running simultaneously.

"""
__restriction = [ 'loader.py', 'scorecard.py', 'dom.py' ]

def __enumerateFiles():
    import os, os.path
    pyfiles = []
    for ( path, dirs, filenames ) in os.walk( "./" ):
        for filename in filenames:
            if filename[ -3: ] == '.py':
                if __restriction.count( filename ) == 0:
                    pyfiles.append( "%s/%s" % ( path, filename ) )
                    pass
                pass
            pass
        pass
    return pyfiles

def __loadModule( filename ):
    import ihooks, os.path
    loader = ihooks.BasicModuleLoader()
    path, file = os.path.split( filename )
    name, ext = os.path.splitext( file )
    mid = loader.find_module_in_dir( name, path )
    if not mid:
        raise ImportError, name
    return loader.load_module( name, mid )

def __loadModuleByName( path, modulename ):
    import ihooks, os.path
    loader = ihooks.BasicModuleLoader()
    mid = loader.find_module_in_dir( modulename, path )
    if not mid:
        raise ImportError, modulename
    return loader.load_module( modulename, mid )

def loadHTML( url, encoding = None ):
    """Load HTML file from website using curl command because Python urllib library
    is too slow."""
    import tempfile, os, re, sys
    url = re.sub( "\"\';", "", url )    
    filename_tmp = tempfile.mktemp( '.html', 'scorecard' )
    command_curl = "/usr/bin/curl \"%s\" -o \"%s\" -s" % ( url, filename_tmp )
    if encoding == None:
        os.system( command_curl )
    else: # convert to utf-8
        os.system( command_curl )
        filename_tmp2 = tempfile.mktemp()
        command_iconv = "/usr/bin/iconv -f %s -t utf-8 \"%s\" > \"%s\"" \
                        % ( encoding, filename_tmp, filename_tmp2 )
        os.system( command_iconv )
        os.unlink( filename_tmp )
        filename_tmp = filename_tmp2
        pass
    data = ""
    if os.path.exists( filename_tmp ) == False: # network error
        raise Exception( "cannot load" )
    
    for line in open( filename_tmp ).readlines():
        data += line
        pass
    os.unlink( filename_tmp )
    return data

def getModuleList():
    """Acquire all modules in the directory. All modules must contain a class inheriting 'ScoreCard' class and newInstance() function which instanciate an object."""
    import scorecard
    import re
    mlist = []
    
    for filename in __enumerateFiles():
        if not re.match( "\\w", filename.split( '/' )[ -1 ] ): continue
        m = __loadModule( filename )
        if not m: continue
        if dir( m ).count( 'newInstance' ) == 0: continue
        obj = None
        try:
            obj = m.newInstance()
            if obj == None: continue
        except:
            continue
        if issubclass( obj.__class__, scorecard.ScoreCard ) == False:
            continue
        mlist.append( ( m, m.__name__, obj.getName(), obj.getSportsCategory(), obj.getLanguage(), obj.getCountry() ) )
        pass
    return mlist

def __output_error( filename ):
    return

if __name__ == "__main__":
    
    import sys
    import tempfile
    import scorecard
    from dom import *

    skin = None

    # module
    if len( sys.argv ) < 2:
        ms = getModuleList()
        ms.sort( lambda x, y: 0 - ( x[ 2 ] < y[ 2 ] ) )
        for m in ms:
            print "\"%s\",\"%s\",\"%s\",\"%s\",\"%s\"" \
                  % ( m[ 1 ], m[ 2 ], m[ 3 ], m[ 4 ], m[ 5 ] )
            pass
        sys.exit()
        pass
    filename_output = None
    module_name = sys.argv[ 1 ]

    # output
    if len( sys.argv ) < 3:
        filename_output = tempfile.mktemp( ".html", "dashsticker" )
    else:
        filename_module = sys.argv[ 2 ]
        sp = filename_module.split( "/" )
        if sp[ -1 ] == "":
            raise Exception()
            sys.exit()
        else:
            fn = sp[ -1 ]
            if fn[ -5 : ] != ".html":
                fn += ".html"
                pass
            filename_output = tempfile.gettempdir() + "/" + fn
            pass
        pass

    # skin
    if len( sys.argv ) >= 4:
        skin = sys.argv[ 3 ]
        pass
    
    ms = getModuleList()
    module = None
    for m in ms:
        if m[ 1 ] == module_name:
            module = m[ 0 ]
            break
        pass
    if module == None:
        __output_error( filename_output )
        raise Exception( "No module" )
        sys.exit()
        pass
    obj = module.newInstance()
    if skin: obj.setSkin( skin )
    
    try:
        if obj != None and obj.isAvailable():
            url = obj.getURL()
            encoding = obj.getEncoding()
            try:
                data = loadHTML( url, encoding )
            except:
                print "NetworkError.html"
                raise
                pass
            kw_start = obj.getStartKeyword()
            kw_end   = obj.getEndKeyword()
            parser = DOMConverter()
            for tag in obj.getIgnorableTags():
                parser.add_ignoring_tag( tag )
                pass
            if kw_start == None or kw_end == None:
                if encoding == None:
                    parser.feed( obj.decode( obj.applyFilter( data ) ) )
                else:
                    for line in data.split( "\n" ):
                        parser.feed( obj.applyFilter( line ) )
                        pass
                    pass
                pass
            else:
                flag_inside = False
                parser.feed( '<html>' )
                for line in re.split( "\n", data ):
                    line = obj.applyFilter( line )
                    if encoding == None:
                        line = obj.decode( line.strip() )
                        pass
                    if flag_inside:
                        if line.count( kw_end ) > 0:
                            flag_inside = False
                            break
                        else:
                            parser.feed( line )
                            pass
                    elif line.count( kw_start ) > 0: flag_inside = True
                    pass
                parser.feed( '</html>' )
                pass
            document = parser.get_document()
            obj.interpreteDocument( document )
            contents = obj.toHTML()
            if contents == None or contents == "":
                uri = "Empty.html"
            else:
                file_out = open( filename_output, "w" )
                file_out.write( contents )
                file_out.close()
                uri = "file://%s" % filename_output
                pass
            print uri
            pass
        else:
            print "Empty.html"
            pass
        pass
    except:
        print "Error.html"
        raise
        pass
    pass

